BEGIN TRAN
BEGIN TRY
    ALTER TABLE [dbo].[DispatchQueue] DROP COLUMN [ContactID];
END TRY
BEGIN CATCH
	PRINT 'Removing ContactID column in DispatchQueue table in EXM Database failed. Consider removing manually.'
	SELECT 
        ERROR_NUMBER() AS ErrorNumber
        ,ERROR_SEVERITY() AS ErrorSeverity
        ,ERROR_STATE() AS ErrorState
        ,ERROR_PROCEDURE() AS ErrorProcedure
        ,ERROR_LINE() AS ErrorLine
        ,ERROR_MESSAGE() AS ErrorMessage;
	
	IF @@TRANCOUNT > 0
        ROLLBACK TRAN;
END CATCH

IF @@TRANCOUNT > 0
    COMMIT TRAN;
GO

BEGIN TRAN
BEGIN TRY
	CREATE TABLE [dbo].[CampaignsNew] (
		[MessageID]             UNIQUEIDENTIFIER NOT NULL,
		[ManagerRootID]         UNIQUEIDENTIFIER NOT NULL,
		[CampaignID]            UNIQUEIDENTIFIER NULL,
		[StartDate]             DATETIME         NULL,
		[EndDate]               DATETIME         NULL,
		[TotalRecipients]       INT              DEFAULT ((0)) NOT NULL,
		[IncludedRecipients]    INT              DEFAULT ((0)) NOT NULL,
		[ExcludedRecipients]    INT              DEFAULT ((0)) NOT NULL,
		[GloballyExcluded]      INT              DEFAULT ((0)) NOT NULL,
		[SentRecipients]        INT              DEFAULT ((0)) NOT NULL,
		[SkippedRecipients]     INT              DEFAULT ((0)) NOT NULL,
		[UnprocessedRecipients] INT              DEFAULT ((0)) NOT NULL,
		[BouncedRecipients]     INT              DEFAULT ((0)) NOT NULL,
		[FailedRecipients]      INT              DEFAULT ((0)) NOT NULL,
		[MessageName]           NVARCHAR (1000)  NULL,
		[Subject]               NVARCHAR (1000)  NULL,
		[MessageType]           INT              DEFAULT ((0)) NOT NULL,
		[Status]                VARCHAR (100)    NULL,
		[CreatedDate]           DATETIME         NULL,
		[ModifiedDate]          DATETIME         NULL,
		[StatusDate]            DATETIME         NULL,
		[ScheduledDate]         DATETIME         NULL,
		[AbTest]                BIT              DEFAULT ((0)) NOT NULL,		
		[Opens]                 INT              DEFAULT ((0)) NOT NULL,
		[UniqueOpens]           INT              DEFAULT ((0)) NOT NULL,
		[OpenRate]              DECIMAL (18, 2)  DEFAULT ((0)) NOT NULL,
		[Clicks]                INT              DEFAULT ((0)) NOT NULL,
		[UniqueClicks]          INT              DEFAULT ((0)) NOT NULL,
		[ClickRate]             DECIMAL (18, 2)  DEFAULT ((0)) NOT NULL,
		[Value]                 INT              DEFAULT ((0)) NOT NULL,
		[ValuePerEmail]         DECIMAL (18, 2)  DEFAULT ((0)) NOT NULL,
		[ValuePerVisit]         DECIMAL (18, 2)  DEFAULT ((0)) NOT NULL,
		[PageBounces]           INT              DEFAULT ((0)) NOT NULL,
		[Unsubscribes]          INT              DEFAULT ((0)) NOT NULL,
		[Spams]                 INT              DEFAULT ((0)) NOT NULL,
		[Browsed]               INT              DEFAULT ((0)) NOT NULL,
		CONSTRAINT [PK_CampaignsNew] PRIMARY KEY CLUSTERED ([MessageID] ASC)
	);

	IF EXISTS (SELECT TOP 1 1 
				FROM   [dbo].[Campaigns])
		BEGIN
			INSERT INTO [dbo].[CampaignsNew] ([CampaignID], [MessageID], [ManagerRootID], [StartDate], [EndDate], [TotalRecipients], [IncludedRecipients], [ExcludedRecipients], [GloballyExcluded])
			SELECT		[ID],
						[MessageID],
						[ManagerRootID],
						[StartDate],
						[EndDate],
						[TotalRecipients],
						[IncludedRecipients],
						[ExcludedRecipients],
						[GloballyExcluded]
			FROM     [dbo].[Campaigns]
			ORDER BY [MessageID] ASC;
		END

	DROP TABLE [dbo].[Campaigns];
	EXECUTE sp_rename N'[dbo].[CampaignsNew]', N'Campaigns';
	EXECUTE sp_rename N'[dbo].[PK_CampaignsNew]', N'PK_Campaigns', N'OBJECT';
	CREATE NONCLUSTERED INDEX [IX_Campaigns_ValuePerVisit] ON [dbo].[Campaigns]([ValuePerVisit] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_ValuePerEmail] ON [dbo].[Campaigns]([ValuePerEmail] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_Value] ON [dbo].[Campaigns]([Value] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_StatusDate] ON [dbo].[Campaigns]([StatusDate] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_ScheduledDate] ON [dbo].[Campaigns]([ScheduledDate] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_MessageType] ON [dbo].[Campaigns]([MessageType] ASC);
	CREATE NONCLUSTERED INDEX [IX_Campaigns_CampaignID] ON [dbo].[Campaigns]([CampaignID] ASC);
END TRY
BEGIN CATCH
	PRINT 'Upgrading to EXM 3.5 Database failed'
	SELECT 
        ERROR_NUMBER() AS ErrorNumber
        ,ERROR_SEVERITY() AS ErrorSeverity
        ,ERROR_STATE() AS ErrorState
        ,ERROR_PROCEDURE() AS ErrorProcedure
        ,ERROR_LINE() AS ErrorLine
        ,ERROR_MESSAGE() AS ErrorMessage;
	
	IF @@TRANCOUNT > 0
        ROLLBACK TRAN;
END CATCH

IF @@TRANCOUNT > 0
    COMMIT TRAN;
GO